import _extends from "@babel/runtime/helpers/extends";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React from 'react';
import Form from '../form';
import PropTypes from 'prop-types';
import RendererContext from '../renderer-context';
import renderForm from './render-form';
import defaultSchemaValidator from '../default-schema-validator';
import SchemaErrorComponent from './schema-error-component';
import defaultValidatorMapper from '../validator-mapper';

var FormRenderer = function FormRenderer(_ref) {
  var componentMapper = _ref.componentMapper,
      FormTemplate = _ref.FormTemplate,
      onSubmit = _ref.onSubmit,
      onCancel = _ref.onCancel,
      _onReset = _ref.onReset,
      clearOnUnmount = _ref.clearOnUnmount,
      subscription = _ref.subscription,
      clearedValue = _ref.clearedValue,
      schema = _ref.schema,
      validatorMapper = _ref.validatorMapper,
      actionMapper = _ref.actionMapper,
      schemaValidatorMapper = _ref.schemaValidatorMapper,
      props = _objectWithoutProperties(_ref, ["componentMapper", "FormTemplate", "onSubmit", "onCancel", "onReset", "clearOnUnmount", "subscription", "clearedValue", "schema", "validatorMapper", "actionMapper", "schemaValidatorMapper"]);

  var schemaError;

  var validatorMapperMerged = _objectSpread(_objectSpread({}, defaultValidatorMapper), validatorMapper);

  try {
    var validatorTypes = Object.keys(validatorMapperMerged);
    var actionTypes = actionMapper ? Object.keys(actionMapper) : [];
    defaultSchemaValidator(schema, componentMapper, validatorTypes, actionTypes, schemaValidatorMapper);
  } catch (error) {
    schemaError = error;
    console.error(error);
    console.log('error: ', error.message);
  }

  if (schemaError) {
    return /*#__PURE__*/React.createElement(SchemaErrorComponent, {
      name: schemaError.name,
      message: schemaError.message
    });
  }

  return /*#__PURE__*/React.createElement(Form, _extends({}, props, {
    clearedValue: clearedValue,
    clearOnUnmount: clearOnUnmount,
    onSubmit: onSubmit,
    subscription: _objectSpread({
      pristine: true,
      submitting: true,
      valid: true
    }, subscription),
    render: function render(_ref2) {
      var _ref2$formOptions = _ref2.formOptions,
          reset = _ref2$formOptions.reset,
          getState = _ref2$formOptions.getState,
          formOptions = _objectWithoutProperties(_ref2$formOptions, ["reset", "getState"]);

      return /*#__PURE__*/React.createElement(RendererContext.Provider, {
        value: {
          componentMapper: componentMapper,
          validatorMapper: validatorMapperMerged,
          actionMapper: actionMapper,
          formOptions: _objectSpread({
            onSubmit: onSubmit,
            onCancel: onCancel ? function () {
              for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
              }

              return onCancel.apply(void 0, [getState().values].concat(args));
            } : undefined,
            onReset: function onReset() {
              _onReset && _onReset.apply(void 0, arguments);
              reset();
            },
            getState: getState,
            reset: reset,
            renderForm: renderForm
          }, formOptions)
        }
      }, /*#__PURE__*/React.createElement(FormTemplate, {
        formFields: renderForm(schema.fields),
        schema: schema
      }));
    }
  }));
};

FormRenderer.propTypes = {
  onSubmit: PropTypes.func.isRequired,
  onCancel: PropTypes.func,
  onReset: PropTypes.func,
  schema: PropTypes.object.isRequired,
  clearOnUnmount: PropTypes.bool,
  subscription: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.bool)),
  clearedValue: PropTypes.any,
  componentMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.oneOfType([PropTypes.node, PropTypes.element, PropTypes.func]))).isRequired,
  FormTemplate: PropTypes.func.isRequired,
  validatorMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
  actionMapper: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
  schemaValidatorMapper: PropTypes.shape({
    components: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
    validators: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func)),
    actions: PropTypes.shape(_defineProperty({}, PropTypes.string, PropTypes.func))
  })
};
FormRenderer.defaultProps = {
  initialValues: {},
  clearOnUnmount: false
};
export default FormRenderer;