"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireDefault(require("react"));

var _form = _interopRequireDefault(require("../form"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _rendererContext = _interopRequireDefault(require("../renderer-context"));

var _renderForm = _interopRequireDefault(require("./render-form"));

var _defaultSchemaValidator = _interopRequireDefault(require("../default-schema-validator"));

var _schemaErrorComponent = _interopRequireDefault(require("./schema-error-component"));

var _validatorMapper = _interopRequireDefault(require("../validator-mapper"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var FormRenderer = function FormRenderer(_ref) {
  var componentMapper = _ref.componentMapper,
      FormTemplate = _ref.FormTemplate,
      onSubmit = _ref.onSubmit,
      onCancel = _ref.onCancel,
      _onReset = _ref.onReset,
      clearOnUnmount = _ref.clearOnUnmount,
      subscription = _ref.subscription,
      clearedValue = _ref.clearedValue,
      schema = _ref.schema,
      validatorMapper = _ref.validatorMapper,
      actionMapper = _ref.actionMapper,
      schemaValidatorMapper = _ref.schemaValidatorMapper,
      props = (0, _objectWithoutProperties2["default"])(_ref, ["componentMapper", "FormTemplate", "onSubmit", "onCancel", "onReset", "clearOnUnmount", "subscription", "clearedValue", "schema", "validatorMapper", "actionMapper", "schemaValidatorMapper"]);
  var schemaError;

  var validatorMapperMerged = _objectSpread(_objectSpread({}, _validatorMapper["default"]), validatorMapper);

  try {
    var validatorTypes = Object.keys(validatorMapperMerged);
    var actionTypes = actionMapper ? Object.keys(actionMapper) : [];
    (0, _defaultSchemaValidator["default"])(schema, componentMapper, validatorTypes, actionTypes, schemaValidatorMapper);
  } catch (error) {
    schemaError = error;
    console.error(error);
    console.log('error: ', error.message);
  }

  if (schemaError) {
    return /*#__PURE__*/_react["default"].createElement(_schemaErrorComponent["default"], {
      name: schemaError.name,
      message: schemaError.message
    });
  }

  return /*#__PURE__*/_react["default"].createElement(_form["default"], (0, _extends2["default"])({}, props, {
    clearedValue: clearedValue,
    clearOnUnmount: clearOnUnmount,
    onSubmit: onSubmit,
    subscription: _objectSpread({
      pristine: true,
      submitting: true,
      valid: true
    }, subscription),
    render: function render(_ref2) {
      var _ref2$formOptions = _ref2.formOptions,
          reset = _ref2$formOptions.reset,
          getState = _ref2$formOptions.getState,
          formOptions = (0, _objectWithoutProperties2["default"])(_ref2$formOptions, ["reset", "getState"]);
      return /*#__PURE__*/_react["default"].createElement(_rendererContext["default"].Provider, {
        value: {
          componentMapper: componentMapper,
          validatorMapper: validatorMapperMerged,
          actionMapper: actionMapper,
          formOptions: _objectSpread({
            onSubmit: onSubmit,
            onCancel: onCancel ? function () {
              for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
              }

              return onCancel.apply(void 0, [getState().values].concat(args));
            } : undefined,
            onReset: function onReset() {
              _onReset && _onReset.apply(void 0, arguments);
              reset();
            },
            getState: getState,
            reset: reset,
            renderForm: _renderForm["default"]
          }, formOptions)
        }
      }, /*#__PURE__*/_react["default"].createElement(FormTemplate, {
        formFields: (0, _renderForm["default"])(schema.fields),
        schema: schema
      }));
    }
  }));
};

FormRenderer.propTypes = {
  onSubmit: _propTypes["default"].func.isRequired,
  onCancel: _propTypes["default"].func,
  onReset: _propTypes["default"].func,
  schema: _propTypes["default"].object.isRequired,
  clearOnUnmount: _propTypes["default"].bool,
  subscription: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].bool)),
  clearedValue: _propTypes["default"].any,
  componentMapper: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].oneOfType([_propTypes["default"].node, _propTypes["default"].element, _propTypes["default"].func]))).isRequired,
  FormTemplate: _propTypes["default"].func.isRequired,
  validatorMapper: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].func)),
  actionMapper: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].func)),
  schemaValidatorMapper: _propTypes["default"].shape({
    components: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].func)),
    validators: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].func)),
    actions: _propTypes["default"].shape((0, _defineProperty2["default"])({}, _propTypes["default"].string, _propTypes["default"].func))
  })
};
FormRenderer.defaultProps = {
  initialValues: {},
  clearOnUnmount: false
};
var _default = FormRenderer;
exports["default"] = _default;