"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _react = require("react");

var _formStateManager = require("@data-driven-forms/form-state-manager");

var _rendererContext = _interopRequireDefault(require("../renderer-context"));

var _assignSpecialType = _interopRequireDefault(require("./assign-special-type"));

var _componentTypes = _interopRequireDefault(require("../component-types"));

var _validatorHelpers = require("./validator-helpers");

var _composeValidators = _interopRequireDefault(require("../compose-validators"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var calculateArrayValidator = function calculateArrayValidator(props, validate, component, validatorMapper) {
  if ((validate || props.dataType) && _componentTypes["default"].FIELD_ARRAY === component) {
    return (0, _validatorHelpers.prepareArrayValidator)((0, _validatorHelpers.getValidate)(validate, props.dataType, validatorMapper));
  }
};

var calculateValidate = function calculateValidate(props, validate, component, validatorMapper) {
  if ((validate || props.dataType) && _componentTypes["default"].FIELD_ARRAY !== component) {
    return (0, _composeValidators["default"])((0, _validatorHelpers.getValidate)(validate, props.dataType, validatorMapper));
  }
};

var init = function init(_ref) {
  var props = _ref.props,
      validate = _ref.validate,
      component = _ref.component,
      validatorMapper = _ref.validatorMapper;
  return {
    arrayValidator: calculateArrayValidator(props, validate, component, validatorMapper),
    validate: calculateValidate(props, validate, component, validatorMapper),
    type: (0, _assignSpecialType["default"])(component)
  };
};

var reducer = function reducer(state, _ref2) {
  var type = _ref2.type,
      specialType = _ref2.specialType,
      validate = _ref2.validate,
      arrayValidator = _ref2.arrayValidator,
      initialValue = _ref2.initialValue;

  switch (type) {
    case 'setType':
      return _objectSpread(_objectSpread({}, state), {}, {
        type: specialType
      });

    case 'setValidators':
      return _objectSpread(_objectSpread({}, state), {}, {
        validate: validate,
        arrayValidator: arrayValidator
      });

    case 'setInitialValue':
      return _objectSpread(_objectSpread({}, state), {}, {
        initialValue: initialValue
      });

    default:
      return state;
  }
};

var createFieldProps = function createFieldProps(name, formOptions) {
  var _ref3 = formOptions.getFieldState(name) || {},
      value = _ref3.value,
      blur = _ref3.blur,
      change = _ref3.change,
      focus = _ref3.focus,
      meta = (0, _objectWithoutProperties2["default"])(_ref3, ["value", "blur", "change", "focus"]);

  return {
    meta: meta,
    input: {
      name: name,
      value: value
    }
  };
};

var useFieldApi = function useFieldApi(_ref4) {
  var name = _ref4.name,
      component = _ref4.component,
      render = _ref4.render,
      validate = _ref4.validate,
      resolveProps = _ref4.resolveProps,
      props = (0, _objectWithoutProperties2["default"])(_ref4, ["name", "component", "render", "validate", "resolveProps"]);

  var _useContext = (0, _react.useContext)(_rendererContext["default"]),
      validatorMapper = _useContext.validatorMapper,
      formOptions = _useContext.formOptions;

  var _ref5 = resolveProps ? resolveProps(props, createFieldProps(name, formOptions), formOptions) || {} : {},
      resolvePropsValidate = _ref5.validate,
      resolvedProps = (0, _objectWithoutProperties2["default"])(_ref5, ["validate"]);

  var finalValidate = resolvePropsValidate || validate;

  var _useReducer = (0, _react.useReducer)(reducer, {
    props: _objectSpread(_objectSpread({}, props), resolvedProps),
    validate: finalValidate,
    component: component,
    validatorMapper: validatorMapper
  }, init),
      _useReducer2 = (0, _slicedToArray2["default"])(_useReducer, 2),
      _useReducer2$ = _useReducer2[0],
      type = _useReducer2$.type,
      stateValidate = _useReducer2$.validate,
      arrayValidator = _useReducer2$.arrayValidator,
      dispatch = _useReducer2[1];

  var mounted = (0, _react.useRef)(false);

  var enhancedProps = _objectSpread(_objectSpread(_objectSpread({
    type: type,
    name: name
  }, props), resolvedProps), stateValidate ? {
    validate: stateValidate
  } : {});

  var fieldProps = (0, _formStateManager.useField)(enhancedProps);
  /** Reinitilize type */

  (0, _react.useEffect)(function () {
    if (mounted.current) {
      var specialType = (0, _assignSpecialType["default"])(component);

      if (specialType !== type) {
        dispatch({
          type: 'setType',
          specialType: specialType
        });
      }
    }
  }, [component]);
  /** Reinitilize array validator/validate */

  (0, _react.useEffect)(function () {
    if (mounted.current) {
      dispatch({
        type: 'setValidators',
        validate: calculateValidate(enhancedProps, finalValidate, component, validatorMapper),
        arrayValidator: calculateArrayValidator(enhancedProps, finalValidate, component, validatorMapper)
      });
    }
    /**
     * We have to stringify the validate array in order to preven infinite looping when validate was passed directly to useFieldApi
     * const x = useFieldApu({name: 'foo', validate: [{type: 'bar'}]}) will trigger infinite looping witouth the serialize.
     * Using stringify is acceptable here since the array is usually very small.
     * If we notice performance hit, we can implement custom hook with a deep equal functionality.
     */

  }, [finalValidate ? JSON.stringify(finalValidate) : false, component, enhancedProps.dataType]);
  (0, _react.useEffect)(function () {
    mounted.current = true;
    return function () {
      mounted.current = false;
    };
  }, []);
  var _initialValue = enhancedProps.initialValue,
      clearOnUnmount = enhancedProps.clearOnUnmount,
      dataType = enhancedProps.dataType,
      clearedValue = enhancedProps.clearedValue,
      _isEqual = enhancedProps.isEqual,
      _validate = enhancedProps.validate,
      _type = enhancedProps.type,
      _initializeOnMount = enhancedProps.initializeOnMount,
      cleanProps = (0, _objectWithoutProperties2["default"])(enhancedProps, ["initialValue", "clearOnUnmount", "dataType", "clearedValue", "isEqual", "validate", "type", "initializeOnMount"]);
  /**
   * construct component props necessary that would live in field provider
   */

  return _objectSpread(_objectSpread(_objectSpread({}, cleanProps), fieldProps), arrayValidator && {
    arrayValidator: arrayValidator
  });
};

var _default = useFieldApi;
exports["default"] = _default;